@@ -52,6 +52,7 @@ enum
   CODEC_CHANGED,
   CHOSE_CANDIDATES,
   STATE_CHANGED,
+  SINK_PAD_READY,
   LAST_SIGNAL
 };
 
@@ -68,6 +69,7 @@ struct _FarsightStreamPrivate
   FarsightMediaType media_type;
   FarsightStreamState state;
   FarsightStreamDirection direction;
+  FarsightStreamDirection current_direction;
   gboolean disposed;
 };
 
@@ -271,6 +273,26 @@ farsight_stream_class_init (FarsightStreamClass *klass)
       farsight_marshal_VOID__INT_INT,
       G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
+  /**
+   * FarsightStream::sink-pad-ready:
+   * @self: #FarsightStream that emmitted the signal
+   * @pad: A GstPad that represents the sink
+   *
+   * This signal is emitted when a sink pad has been created on the Farsight
+   * stream bin.
+   *
+   * Note : This signal is only valid when a user-defined pipeline has been
+   * provided by the user to farsight using #farsight_stream_set_pipeline.
+   */
+  signals[SINK_PAD_READY] = g_signal_new ("sink-pad-ready",
+      G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      0,
+      NULL,
+      NULL,
+      g_cclosure_marshal_VOID__POINTER,
+      G_TYPE_NONE, 1, G_TYPE_POINTER);
+
   gobject_class->dispose = farsight_stream_dispose;
   gobject_class->finalize = farsight_stream_finalize;
 
@@ -284,7 +306,8 @@ farsight_stream_init (FarsightStream *self)
   self->priv = FARSIGHT_STREAM_GET_PRIVATE (self);
   self->priv->disposed = FALSE;
   self->priv->state = FARSIGHT_STREAM_STATE_DISCONNECTED;
-  self->priv->direction = FARSIGHT_STREAM_DIRECTION_NONE;
+  self->priv->direction = FARSIGHT_STREAM_DIRECTION_BOTH;
+  self->priv->current_direction = FARSIGHT_STREAM_DIRECTION_NONE;
 }
 
 static void
@@ -469,7 +492,7 @@ farsight_stream_remove_remote_candidate (FarsightStream *self,
  * Sets the given candidate pair as the active candidate pair,
  * and emits the "new-active-candidate-pair" signal.
  *
- * Returns: TRUE if sucessfully set the active candidate pair
+ * Returns: %TRUE if sucessfully set the active candidate pair
  */
 gboolean
 farsight_stream_set_active_candidate_pair (FarsightStream *self,
@@ -555,20 +578,24 @@ farsight_stream_get_local_codecs (FarsightStream *self)
  * @codecs: #GList of #FarsightCodec 
  *
  * Set the selection of codecs that the remote end claims to understand
+ *
+ * Return value: %TRUE if it succeeds and %FALSE if there is no intersection
+ * between remote and local codecs.
  */
-void
+gboolean
 farsight_stream_set_remote_codecs (FarsightStream *self,
                                    const GList *codecs)
 {
   FarsightStreamClass *klass = FARSIGHT_STREAM_GET_CLASS (self);
-  g_return_if_fail (g_type_is_a (G_OBJECT_TYPE (self), FARSIGHT_TYPE_STREAM));
-  g_return_if_fail (codecs != NULL);
+  g_return_val_if_fail (FARSIGHT_IS_STREAM(self), FALSE);
+  g_return_val_if_fail (codecs != NULL, FALSE);
 
   if (klass->set_remote_codecs) {
-    klass->set_remote_codecs (self, codecs);
+    return klass->set_remote_codecs (self, codecs);
   } else {
-    g_warning ("set_remote_codecs not defined for %s", 
+    g_warning ("set_remote_codecs not defined for %s",
         G_OBJECT_TYPE_NAME (self));
+    return FALSE;
   }
 }
 
@@ -601,13 +628,13 @@ farsight_stream_get_codec_intersection (FarsightStream *self)
 /**
  * farsight_stream_set_codec_preference_list:
  * @self: a #FarsightStream
- * @codec_pref: an array of #FarsightCodecPreference
+ * @codec_pref: a GArray of #FarsightCodecPreference
  *
  * Set a list of prefered codecs (optional)
  *
  */
 void farsight_stream_set_codec_preference_list (FarsightStream *self,
-    FarsightCodecPreference codec_pref[])
+    const GArray *codec_pref)
 {
   FarsightStreamClass *klass = FARSIGHT_STREAM_GET_CLASS (self);
   g_return_if_fail (g_type_is_a (G_OBJECT_TYPE (self), FARSIGHT_TYPE_STREAM));
@@ -676,7 +703,7 @@ farsight_stream_get_active_codec (FarsightStream *self)
  * replace the old one. If there is a user specified pipeline, the sink won't be
  * added into the Farsight bin.
  *
- * Returns: TRUE if sucessful
+ * Returns: %TRUE if sucessful
  */
 gboolean
 farsight_stream_set_sink (FarsightStream *self,
@@ -702,7 +729,7 @@ farsight_stream_set_sink (FarsightStream *self,
  * 
  * Set the GStreamer sink link filter for this stream
  *
- * Returns: TRUE if sucessful
+ * Returns: %TRUE if sucessful
  **/
 gboolean 
 farsight_stream_set_sink_filter (FarsightStream *self, 
@@ -755,7 +782,7 @@ farsight_stream_get_sink (FarsightStream *self)
  * parent, it will be added into the Farsight stream bin. So if you need to set
  * a parent for the source, do it before calling this function.
  *
- * Returns: TRUE if sucessful
+ * Returns: %TRUE if sucessful
  **/
 gboolean
 farsight_stream_set_source (FarsightStream *self,
@@ -783,7 +810,7 @@ farsight_stream_set_source (FarsightStream *self,
  * 
  * Set the GStreamer source link filter for this stream
  *
- * Returns: TRUE if sucessful
+ * Returns: %TRUE if sucessful
  **/
 gboolean 
 farsight_stream_set_source_filter (FarsightStream *self, 
@@ -844,7 +871,7 @@ FarsightStreamState farsight_stream_get_state (FarsightStream *self)
  * farsight_stream_get_direction:
  * @self: a #FarsightStream
  *
- * Get the #FarsightStream direction
+ * Get the #FarsightStream requested direction
  *
  * Return value: #FarsightStreamState direction
  */
@@ -853,11 +880,24 @@ FarsightStreamDirection farsight_stream_get_direction (FarsightStream *self)
   return self->priv->direction;
 }
 
+/**
+ * farsight_stream_get_current_direction:
+ * @self: a #FarsightStream
+ *
+ * Get the #FarsightStream current direction
+ *
+ * Return value: #FarsightStreamState direction
+ */
+FarsightStreamDirection farsight_stream_get_current_direction (FarsightStream *self)
+{
+  return self->priv->current_direction;
+}
+
 /**
  * farsight_stream_get_media_type:
  * @self: a #FarsightStream
  *
- * Get the #FarsightStream direction
+ * Get the #FarsightStream media type
  *
  * Return value: #FarsightMediaType media type
  */
@@ -905,7 +945,7 @@ GstElement * farsight_stream_get_pipeline (FarsightStream *self)
  * NOTE: This has to be called before farsight_stream_prepare_transports() in
  * order to be used.
  *
- * Return value: TRUE if successful
+ * Return value: %TRUE if successful
  *
  */
 gboolean farsight_stream_set_pipeline (FarsightStream *self, GstElement *pipeline)
@@ -930,7 +970,7 @@ gboolean farsight_stream_set_pipeline (FarsightStream *self, GstElement *pipelin
  *
  * Start the #FarsightStream streaming to the network
  *
- * Return value: TRUE if sucessful
+ * Return value: %TRUE if sucessful
  */
 gboolean
 farsight_stream_start (FarsightStream *self)
@@ -976,7 +1016,7 @@ farsight_stream_stop (FarsightStream *self)
  * This function will tell Farsight to stop sending. This will block the source
  * pad that is given to Farsight.
  *
- * Return value: TRUE if successful
+ * Return value: %TRUE if successful
  *
  */
 gboolean farsight_stream_set_sending (FarsightStream *self, gboolean sending)
@@ -1089,6 +1129,7 @@ farsight_stream_signal_codec_changed (FarsightStream *self,
 {
   g_signal_emit (self, signals[CODEC_CHANGED], 0, codec_id);
 }
+
 /**
  * farsight_stream_signal_state_changed:
  * @self: a #FarsightStream
@@ -1103,12 +1144,28 @@ void farsight_stream_signal_state_changed (FarsightStream *self,
                                            FarsightStreamState state,
                                            FarsightStreamDirection direction)
 {
-  if (self->priv->state == state)
+  if (self->priv->state == state && self->priv->current_direction == direction)
     return;
   self->priv->state = state;
+  self->priv->current_direction = direction;
   g_signal_emit (self, signals[STATE_CHANGED], 0, state, direction);
 }
 
+/**
+ * farsight_stream_signal_sink_pad_ready:
+ * @self: a #FarsightStream
+ * @pad: A GstPad that represents the sink
+ *
+ * Used by subclasses of #FarsightStream to emit an
+ * <link linkend="FarsightStream-sink-pad-ready">sink-pad-ready
+ * </link> signal
+ */
+void farsight_stream_signal_sink_pad_ready (FarsightStream *self,
+                                            GstPad *pad)
+{
+  g_signal_emit (self, signals[SINK_PAD_READY], 0, pad);
+}
+
 static void
 farsight_stream_set_property (GObject *object,
                               guint prop_id,
@@ -1158,3 +1215,141 @@ farsight_stream_get_property (GObject *object,
       break;
   }
 }
+
+
+/**
+ * farsight_stream_start_telephony_event:
+ * @self: a #FarsightStream
+ * @ev: A #FarsightStreamDTMFEvent or another number defined at
+ * http://www.iana.org/assignments/audio-telephone-event-registry
+ * @volume: The volume in dBm0 without the negative sign. Should be between
+ * 0 and 36. Higher values mean lower volume
+ *
+ * This function will start sending a telephony event (such as a DTMF
+ * tone) on the #FarsightStream, you have to call the function
+ * farsight_stream_stop_telephony_event() to stop it. 
+ * This function will use any available method, if you want to use a specific
+ * method only, use farsight_stream_start_telephony_event_full()
+ *
+ * Return value: %TRUE if sucessful, it can return %FALSE if the #FarsightStream
+ * does not support this telephony event.
+ */
+gboolean
+farsight_stream_start_telephony_event (FarsightStream *self,
+                                       guint8 ev,
+                                       guint8 volume)
+{
+  return farsight_stream_start_telephony_event_full (self, ev, volume,
+      FARSIGHT_DTMF_METHOD_AUTO);
+}
+
+
+/**
+ * farsight_stream_stop_telephony_event:
+ * @self: a #FarsightStream
+ *
+ * This function will stop sending a telephony event started by
+ * farsight_stream_start_telephony_event(). If the event was being sent
+ * for less than 50ms, it will be sent for 50ms minimum. If the
+ * duration was a positive and the event is not over, it will cut it
+ * short.
+ *
+ * Return value: %TRUE if sucessful, it can return %FALSE if the #FarsightStream
+ * does not support telephony events or if no telephony event is being sent
+ */
+gboolean
+farsight_stream_stop_telephony_event (FarsightStream *self)
+{
+  return farsight_stream_stop_telephony_event_full (self, FARSIGHT_DTMF_METHOD_AUTO);
+}
+
+
+/**
+ * farsight_stream_start_telephony_event_full:
+ * @self: a #FarsightStream
+ * @ev: A #FarsightStreamDTMFEvent or another number defined at
+ * http://www.iana.org/assignments/audio-telephone-event-registry
+ * @volume: The volume in dBm0 without the negative sign. Should be between
+ * 0 and 36. Higher values mean lower volume
+ * @type: The way the event should be sent
+ * @method: The method used to send the event
+ *
+ * This function will start sending a telephony event (such as a DTMF
+ * tone) on the #FarsightStream, you have to call the function
+ * farsight_stream_stop_telephony_event_full() to stop it.
+ *
+ * Return value: %TRUE if sucessful, it can return %FALSE if the #FarsightStream
+ * does not support this telephony event.
+ */
+gboolean
+farsight_stream_start_telephony_event_full (FarsightStream *self,
+    guint8 ev, guint8 volume, FarsightStreamDTMFMethod method)
+{
+  FarsightStreamClass *klass = FARSIGHT_STREAM_GET_CLASS (self);
+  g_return_val_if_fail (g_type_is_a (G_OBJECT_TYPE (self),
+              FARSIGHT_TYPE_STREAM), FALSE);
+
+  if (klass->start_telephony_event) {
+    return klass->start_telephony_event (self, ev, volume, method);
+  }
+
+  return FALSE;
+}
+
+
+/**
+ * farsight_stream_stop_telephony_event_full:
+ * @self: a #FarsightStream
+ * @method: The method used to send the event, this MUST match the parameter
+ * passed to farsight_stream_start_telephony_event_full().
+ *
+ * This function will stop sending a telephony event started by
+ * farsight_stream_start_telephony_event_full(). If the event was being sent
+ * for less than 50ms, it will be sent for 50ms minimum. If the
+ * duration was a positive and the event is not over, it will cut it
+ * short. The type parameters has to be the same type that was passed to
+ *
+ * Return value: %TRUE if sucessful, it can return %FALSE if the #FarsightStream
+ * does not support telephony events or if no telephony event is being sent
+ */
+gboolean
+farsight_stream_stop_telephony_event_full (FarsightStream *self,
+                                           FarsightStreamDTMFMethod method)
+{
+  FarsightStreamClass *klass = FARSIGHT_STREAM_GET_CLASS (self);
+  g_return_val_if_fail (g_type_is_a (G_OBJECT_TYPE (self),
+              FARSIGHT_TYPE_STREAM), FALSE);
+
+  if (klass->stop_telephony_event) {
+    return klass->stop_telephony_event (self, method);
+  }
+
+  return FALSE;
+}
+
+/**
+ * farsight_stream_preload_receive_pipeline
+ * @self: a #FarsightStream
+ * @payload_type: The payload type for which to preload the codec
+ *
+ * This function will preload a codec instead of loading it when the first
+ * codec arrives. It is particularly useful with DSP and other slow to load
+ * and initialise codecs.
+ *
+ * Return value: TRUE if sucessful, it can return FALSE if the #FarsightStream
+ * is already receiving packets or if the codec can not be preloaded.
+ */
+
+gboolean farsight_stream_preload_receive_pipeline (
+    FarsightStream *self, gint payload_type)
+{
+  FarsightStreamClass *klass = FARSIGHT_STREAM_GET_CLASS (self);
+  g_return_val_if_fail (g_type_is_a (G_OBJECT_TYPE (self),
+          FARSIGHT_TYPE_STREAM), FALSE);
+
+  if (klass->preload_receive_pipeline) {
+    return klass->preload_receive_pipeline (self, payload_type);
+  }
+
+  return FALSE;
+}
