@@ -69,6 +69,8 @@ struct _FarsightUDPTransmitterPrivate
   guint stun_port;
   GIOChannel *stun_ioc; /* GIOChannel used for the STUN */
   guint stun_timeoutid; /* id of the GSource for the STUN timeout*/
+  guint stun_source;
+  guint stun_source_error;
 };
 
 
@@ -370,14 +372,21 @@ farsight_udp_transmitter_dispose (GObject *object)
     self->priv->stun_timeoutid = 0;
   }
 
+  if (self->priv->stun_source) {
+    g_source_remove (self->priv->stun_source);
+    self->priv->stun_source = 0;
+  }
+
+  if (self->priv->stun_source_error) {
+    g_source_remove (self->priv->stun_source_error);
+    self->priv->stun_source_error = 0;
+  }
+
   if (self->priv->stun_ioc) {
     g_io_channel_unref (self->priv->stun_ioc);
     self->priv->stun_ioc = NULL;
   }
 
-  /* Remove all leftover event sources related to the transmitter */
-  while (g_source_remove_by_user_data (self)) {}
-
   self->priv->disposed = TRUE;
 
   /* chain up to parent */
@@ -588,21 +597,24 @@ farsight_udp_transmitter_start_stun (FarsightUDPTransmitter *transmitter)
   g_free (packed);
   stun_message_free (msg);
 
-  transmitter->priv->stun_ioc = 
+  transmitter->priv->stun_ioc =
     g_io_channel_unix_new (transmitter->priv->local_socket);
 
-  g_io_add_watch (transmitter->priv->stun_ioc, G_IO_IN, 
-      farsight_udp_transmitter_handle_stun, transmitter);
-  
-  g_io_add_watch (transmitter->priv->stun_ioc, G_IO_ERR|G_IO_HUP|G_IO_NVAL, 
-      farsight_udp_transmitter_handle_stun_error, transmitter);
+  transmitter->priv->stun_source =
+      g_io_add_watch (transmitter->priv->stun_ioc, G_IO_IN,
+          farsight_udp_transmitter_handle_stun, transmitter);
+
+  transmitter->priv->stun_source_error =
+      g_io_add_watch (transmitter->priv->stun_ioc, G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+          farsight_udp_transmitter_handle_stun_error, transmitter);
 
-  
   transmitter->priv->stun_timeoutid = g_timeout_add (
       DEFAULT_STUN_TIMEOUT * 1000, farsight_udp_transmitter_stun_timeout, 
       transmitter);
 
-  if (!transmitter->priv->stun_timeoutid) {
+  if (!transmitter->priv->stun_source ||
+      !transmitter->priv->stun_source_error ||
+      !transmitter->priv->stun_timeoutid) {
     farsight_udp_transmitter_stop_stun (transmitter);
     return FALSE;
   }
@@ -614,17 +626,25 @@ farsight_udp_transmitter_start_stun (FarsightUDPTransmitter *transmitter)
 static void
 farsight_udp_transmitter_stop_stun (FarsightUDPTransmitter *transmitter)
 {
-  if (!transmitter)
-    return;
-  if (!FARSIGHT_IS_UDP_TRANSMITTER (transmitter))
-    return;
+  g_return_if_fail (transmitter);
+  g_return_if_fail (FARSIGHT_IS_UDP_TRANSMITTER (transmitter));
 
 
   if (transmitter->priv->stun_ioc) {
     g_io_channel_unref (transmitter->priv->stun_ioc);
     transmitter->priv->stun_ioc = NULL;
   }
-  
+
+  if (transmitter->priv->stun_source) {
+    g_source_remove (transmitter->priv->stun_source);
+    transmitter->priv->stun_source = 0;
+  }
+
+  if (transmitter->priv->stun_source_error) {
+    g_source_remove (transmitter->priv->stun_source_error);
+    transmitter->priv->stun_source_error = 0;
+  }
+
   if (transmitter->priv->stun_timeoutid) {
     g_source_remove (transmitter->priv->stun_timeoutid);
     transmitter->priv->stun_timeoutid = 0;
@@ -697,8 +717,7 @@ farsight_udp_transmitter_handle_stun (GIOChannel *source,
     }
   }
 
-  if (msg)
-    stun_message_free (msg);
+  stun_message_free (msg);
 
   if (!trans)
     g_debug ("Got STUN reply without a mapped-address");
@@ -727,8 +746,6 @@ farsight_udp_transmitter_handle_stun_error (GIOChannel *source,
       FARSIGHT_TRANSMITTER_ERROR_UNKNOWN,
       "Socket error while waiting for STUN reply");
 
-  farsight_udp_transmitter_use_local_ips (transmitter);
-    
   return FALSE;
 }
 
