@@ -136,8 +136,8 @@ state_changed (FarsightStream * stream,
 void
 setup_send (gchar * host, gint port)
 {
-  send_sock = socket (AF_INET, SOCK_DGRAM, 0);
   struct hostent *rx_addr = gethostbyname (host);
+  send_sock = socket (AF_INET, SOCK_DGRAM, 0);
 
   serv_addr.sin_family = AF_INET;
   bcopy (rx_addr->h_addr, &serv_addr.sin_addr, rx_addr->h_length);
@@ -149,9 +149,9 @@ GIOChannel *
 setup_recv (gint port)
 {
   int sockfd = socket (AF_INET, SOCK_DGRAM, 0);
+  struct sockaddr_in dest_addr;
   g_message (G_STRFUNC);
 
-  struct sockaddr_in dest_addr;
 
   bzero (&dest_addr, sizeof (dest_addr));
   dest_addr.sin_family = AF_INET;
@@ -219,10 +219,10 @@ do_handshake(gint sockfd)
   gint type;
   socklen_t fromlen;
   struct sockaddr_in from_addr;
+  gchar buf[1500] = { 0 };
 
   send_ready_byte();
   /* wait for reply */
-  gchar buf[1500] = { 0 };
   fromlen = sizeof (struct sockaddr_in);
 
   g_message ("waiting for ready byte");
@@ -306,6 +306,8 @@ add_remote_codec (FarsightStream *stream, gint pt, gchar *encoding_name,
   static GList *remote_codecs_audio = NULL;
   static GList *remote_codecs_video = NULL;
   GList *lp = NULL;
+  FarsightCodec *codec;
+
   if (g_ascii_strcasecmp (encoding_name, "LAST") == 0)
   {
     if (media_type == FARSIGHT_MEDIA_TYPE_AUDIO)
@@ -319,7 +321,7 @@ add_remote_codec (FarsightStream *stream, gint pt, gchar *encoding_name,
     return;
   }
 
-  FarsightCodec *codec = g_new0 (FarsightCodec, 1);
+  codec = g_new0 (FarsightCodec, 1);
 
   codec->id = pt;
   codec->encoding_name = strdup (encoding_name);
@@ -353,11 +355,14 @@ static void
 add_remote_candidate (FarsightStream * stream, gchar * id, gchar * ip,
     gint port, gchar * username, gchar * password)
 {
+  FarsightTransportInfo *trans;
+  GList *candidate_glist = NULL;
+
   //g_print(">>> Adding remote candidate: %s\n", id);
   g_return_if_fail (stream != NULL);
 
   /* let's create our candidate from ip:port given on command line */
-  FarsightTransportInfo *trans = g_new0 (FarsightTransportInfo, 1);
+  trans = g_new0 (FarsightTransportInfo, 1);
 
   trans->candidate_id = g_strdup (id);
   trans->component = 1;
@@ -371,8 +376,6 @@ add_remote_candidate (FarsightStream * stream, gchar * id, gchar * ip,
   trans->username = g_strdup (username);
   trans->password = g_strdup (password);
 
-  GList *candidate_glist = NULL;
-
   candidate_glist = g_list_append (candidate_glist, trans);
 
   farsight_stream_add_remote_candidate (stream, candidate_glist);
@@ -483,6 +486,7 @@ main (int argc, char **argv)
   char *sinktype;
   GIOChannel *recv_chan;
   gint mode;
+  FarsightSession *session;
 
   if (argc != 4 && argc != 5 && argc != 6)
   {
@@ -518,8 +522,6 @@ main (int argc, char **argv)
 
   g_io_add_watch (recv_chan, G_IO_IN | G_IO_PRI, receive_loop, NULL);
 
-  FarsightSession *session;
-
   session = setup_rtp_session ();
 
   if (mode == MODE_AUDIO || mode == MODE_DUPLEX)
@@ -535,10 +537,10 @@ main (int argc, char **argv)
    * send pipeline change from the start */
   if (audio_stream)
   {
-    farsight_stream_set_active_codec (audio_stream, 8);
-
     GstElement *alsasrc, *alsasink;
 
+    farsight_stream_set_active_codec (audio_stream, 8);
+
     alsasrc = gst_element_factory_make ("audiotestsrc", "src");
     alsasink = gst_element_factory_make ("alsasink", "alsasink");
 
@@ -562,6 +564,7 @@ main (int argc, char **argv)
     GstElement *videosrc, *videosink, *videoscale, *colorspace, *ximagesink,
                *glimagesink;
     GstPad *pad;
+    GstCaps *src_filter;
 
     videosrc = gst_element_factory_make("videotestsrc", NULL);
     g_object_set (G_OBJECT (videosrc), "is-live", TRUE, NULL);
@@ -610,7 +613,7 @@ main (int argc, char **argv)
       gst_object_unref (GST_OBJECT (pad));
     }
 
-    GstCaps *src_filter = gst_caps_new_simple ("video/x-raw-yuv", "width",
+    src_filter = gst_caps_new_simple ("video/x-raw-yuv", "width",
         G_TYPE_INT, 352, "height", G_TYPE_INT, 288, "framerate",
         GST_TYPE_FRACTION, 15,1, NULL);
     farsight_stream_set_source_filter (video_stream, src_filter);
@@ -621,10 +624,11 @@ main (int argc, char **argv)
     /* let's connect the the prepare-xwindow-id bus message sent by xvimagesink */
     if (argc == 6)
     {
-      g_debug ("setting up xoverlay bus callback");
       GstBus *bus;
       GstElement *pipeline = farsight_stream_get_pipeline (video_stream);
       gulong xid = atol (argv[5]);
+
+      g_debug ("setting up xoverlay bus callback");
       bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
       gst_bus_set_sync_handler (bus, (GstBusSyncHandler) set_xoverlay, &xid);
     }
@@ -664,6 +668,9 @@ setup_rtp_stream (FarsightSession * session, FarsightMediaType type)
 
   stream = farsight_session_create_stream (session,
       type, FARSIGHT_STREAM_DIRECTION_BOTH);
+
+  g_object_set (G_OBJECT (stream), "transmitter", "libjingle", NULL);
+
   g_signal_connect (G_OBJECT (stream), "error",
       G_CALLBACK (stream_error), NULL);
   g_signal_connect (G_OBJECT (stream), "new-active-candidate-pair",
